@@ -88,7 +88,9 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
     #: The name of the image.
     name = resource.Body('name')
     #: The ID of the owner, or project, of the image.
-    owner_id = resource.Body('owner')
+    owner = resource.Body('owner', alias='owner_id')
+    #: The ID of the owner, or project, of the image. (backwards compat)
+    owner_id = resource.Body('owner', alias='owner')
     # TODO(mordred) This is not how this works in v2. I mean, it's how it
     # should work, but it's not. We need to fix properties. They work right
     # in shade, so we can draw some logic from there.
@@ -285,7 +287,7 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
         session.post(url, json=json, headers=headers)
 
     def _prepare_request(self, requires_id=None, prepend_key=False,
-                         patch=False, base_path=None):
+                         patch=False, base_path=None, **kwargs):
         request = super(Image, self)._prepare_request(requires_id=requires_id,
                                                       prepend_key=prepend_key,
                                                       patch=patch,
