@@ -11,12 +11,11 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import io
 import operator
 import tempfile
 import uuid
 
-import six
-
 from openstack import exceptions
 from openstack.cloud import exc
 from openstack.cloud import meta
@@ -75,7 +74,7 @@ class TestImage(BaseTestImage):
                           self.cloud.download_image, self.image_name)
 
     def test_download_image_two_outputs(self):
-        fake_fd = six.BytesIO()
+        fake_fd = io.BytesIO()
         self.assertRaises(exc.OpenStackCloudException,
                           self.cloud.download_image, self.image_name,
                           output_path='fake_path', output_file=fake_fd)
@@ -120,7 +119,7 @@ class TestImage(BaseTestImage):
 
     def test_download_image_with_fd(self):
         self._register_image_mocks()
-        output_file = six.BytesIO()
+        output_file = io.BytesIO()
         self.cloud.download_image(self.image_name, output_file=output_file)
         output_file.seek(0)
         self.assertEqual(output_file.read(), self.output)
@@ -321,7 +320,21 @@ class TestImage(BaseTestImage):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'image', append=['images'], base_url_append='v2'),
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='POST',
                  uri=self.get_mock_url(
@@ -356,16 +369,17 @@ class TestImage(BaseTestImage):
             dict(method='GET',
                  uri=self.get_mock_url(
                      'image', append=['images'], base_url_append='v2'),
+                 complete_qs=True,
                  json=self.fake_search_return)
         ])
 
         self.cloud.create_image(
             self.image_name, self.imagefile.name, wait=True, timeout=1,
             tags=['tag1', 'tag2'],
-            is_public=False)
+            is_public=False, validate_checksum=True)
 
         self.assert_calls()
-        self.assertEqual(self.adapter.request_history[5].text.read(),
+        self.assertEqual(self.adapter.request_history[7].text.read(),
                          self.output)
 
     def test_create_image_task(self):
@@ -390,7 +404,21 @@ class TestImage(BaseTestImage):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'image', append=['images'], base_url_append='v2'),
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='HEAD',
                  uri='{endpoint}/{container}'.format(
@@ -517,13 +545,15 @@ class TestImage(BaseTestImage):
             dict(method='GET',
                  uri=self.get_mock_url(
                      'image', append=['images'], base_url_append='v2'),
+                 complete_qs=True,
                  json=self.fake_search_return)
         ])
 
         self.cloud.create_image(
             self.image_name, self.imagefile.name, wait=True, timeout=1,
             disk_format='vhd', container_format='ovf',
-            is_public=False, container=self.container_name)
+            is_public=False, validate_checksum=True,
+            container=self.container_name)
 
         self.assert_calls()
 
@@ -666,7 +696,7 @@ class TestImage(BaseTestImage):
         imagefile.close()
         self.cloud.create_image(
             name, imagefile.name, wait=True, timeout=1,
-            is_public=False, **kwargs)
+            is_public=False, validate_checksum=True, **kwargs)
 
     def test_create_image_put_v1(self):
         self.cloud.config.config['image_api_version'] = '1'
@@ -686,7 +716,11 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v1/images/detail',
+                 uri='https://image.example.com/v1/images/' + self.image_name,
+                 status_code=404),
+            dict(method='GET',
+                 uri='https://image.example.com/v1/images/detail?name='
+                 + self.image_name,
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v1/images',
@@ -726,7 +760,11 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v1/images/detail',
+                 uri='https://image.example.com/v1/images/' + self.image_name,
+                 status_code=404),
+            dict(method='GET',
+                 uri='https://image.example.com/v1/images/detail?name='
+                 + self.image_name,
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v1/images',
@@ -792,7 +830,22 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v2/images',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v2/images',
@@ -828,10 +881,6 @@ class TestImage(BaseTestImage):
         fake_image['owner_specified.openstack.sha256'] = 'b'
 
         self.register_uris([
-            dict(method='GET',
-                 uri=self.get_mock_url(
-                     'image', append=['images'], base_url_append='v2'),
-                 json={'images': []}),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'image', append=['images'], base_url_append='v2'),
@@ -870,7 +919,8 @@ class TestImage(BaseTestImage):
             exceptions.SDKException,
             self.cloud.create_image,
             self.image_name, self.imagefile.name,
-            is_public=False, md5='a', sha256='b'
+            is_public=False, md5='a', sha256='b',
+            allow_duplicates=True, validate_checksum=True
         )
 
         self.assert_calls()
@@ -878,15 +928,10 @@ class TestImage(BaseTestImage):
     def test_create_image_put_bad_int(self):
         self.cloud.image_api_use_tasks = False
 
-        self.register_uris([
-            dict(method='GET',
-                 uri='https://image.example.com/v2/images',
-                 json={'images': []}),
-        ])
-
         self.assertRaises(
             exc.OpenStackCloudException,
             self._call_create_image, self.image_name,
+            allow_duplicates=True,
             min_disk='fish', min_ram=0)
 
         self.assert_calls()
@@ -910,7 +955,22 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v2/images',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v2/images',
@@ -931,6 +991,7 @@ class TestImage(BaseTestImage):
                  json=ret),
             dict(method='GET',
                  uri='https://image.example.com/v2/images',
+                 complete_qs=True,
                  json={'images': [ret]}),
         ])
 
@@ -959,7 +1020,22 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v2/images',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v2/images',
@@ -980,6 +1056,7 @@ class TestImage(BaseTestImage):
                  json=ret),
             dict(method='GET',
                  uri='https://image.example.com/v2/images',
+                 complete_qs=True,
                  json={'images': [ret]}),
         ])
 
@@ -1009,7 +1086,22 @@ class TestImage(BaseTestImage):
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v2/images',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_name],
+                     base_url_append='v2'),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['name=' + self.image_name]),
+                 validate=dict(),
+                 json={'images': []}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'],
+                     base_url_append='v2',
+                     qs_elements=['os_hidden=True']),
                  json={'images': []}),
             dict(method='POST',
                  uri='https://image.example.com/v2/images',
@@ -1030,6 +1122,7 @@ class TestImage(BaseTestImage):
                  json=ret),
             dict(method='GET',
                  uri='https://image.example.com/v2/images',
+                 complete_qs=True,
                  json={'images': [ret]}),
         ])
 
