@@ -15,10 +15,11 @@ import tempfile
 import testtools
 
 import openstack.cloud
-from openstack.cloud import meta
 from openstack.tests import fakes
 from openstack.tests.unit import base
 
+from openstack.orchestration.v1 import stack
+
 
 class TestStack(base.TestCase):
 
@@ -44,7 +45,8 @@ class TestStack(base.TestCase):
         ])
         stacks = self.cloud.list_stacks()
         self.assertEqual(
-            [f.toDict() for f in self.cloud._normalize_stacks(fake_stacks)],
+            [f.toDict() for f in self.cloud._normalize_stacks(
+                stack.Stack(**st) for st in fake_stacks)],
             [f.toDict() for f in stacks])
 
         self.assert_calls()
@@ -76,7 +78,8 @@ class TestStack(base.TestCase):
         ])
         stacks = self.cloud.search_stacks()
         self.assertEqual(
-            self.cloud._normalize_stacks(meta.obj_list_to_munch(fake_stacks)),
+            self.cloud._normalize_stacks(
+                stack.Stack(**st) for st in fake_stacks),
             stacks)
         self.assert_calls()
 
@@ -98,7 +101,7 @@ class TestStack(base.TestCase):
         stacks = self.cloud.search_stacks(filters=filters)
         self.assertEqual(
             self.cloud._normalize_stacks(
-                meta.obj_list_to_munch(fake_stacks[1:])),
+                stack.Stack(**st) for st in fake_stacks[1:]),
             stacks)
         self.assert_calls()
 
@@ -114,20 +117,22 @@ class TestStack(base.TestCase):
             self.cloud.search_stacks()
 
     def test_delete_stack(self):
+        resolve = 'resolve_outputs=False'
         self.register_uris([
             dict(method='GET',
-                 uri='{endpoint}/stacks/{name}'.format(
+                 uri='{endpoint}/stacks/{name}?{resolve}'.format(
                      endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                     name=self.stack_name),
+                     name=self.stack_name, resolve=resolve),
                  status_code=302,
                  headers=dict(
-                     location='{endpoint}/stacks/{name}/{id}'.format(
+                     location='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                          endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                         id=self.stack_id, name=self.stack_name))),
+                         id=self.stack_id, name=self.stack_name,
+                         resolve=resolve))),
             dict(method='GET',
-                 uri='{endpoint}/stacks/{name}/{id}'.format(
+                 uri='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                      endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                     id=self.stack_id, name=self.stack_name),
+                     id=self.stack_id, name=self.stack_name, resolve=resolve),
                  json={"stack": self.stack}),
             dict(method='DELETE',
                  uri='{endpoint}/stacks/{id}'.format(
@@ -138,30 +143,33 @@ class TestStack(base.TestCase):
         self.assert_calls()
 
     def test_delete_stack_not_found(self):
+        resolve = 'resolve_outputs=False'
         self.register_uris([
             dict(method='GET',
-                 uri='{endpoint}/stacks/stack_name'.format(
-                     endpoint=fakes.ORCHESTRATION_ENDPOINT),
+                 uri='{endpoint}/stacks/stack_name?{resolve}'.format(
+                     endpoint=fakes.ORCHESTRATION_ENDPOINT, resolve=resolve),
                  status_code=404),
         ])
         self.assertFalse(self.cloud.delete_stack('stack_name'))
         self.assert_calls()
 
     def test_delete_stack_exception(self):
+        resolve = 'resolve_outputs=False'
         self.register_uris([
             dict(method='GET',
-                 uri='{endpoint}/stacks/{id}'.format(
+                 uri='{endpoint}/stacks/{id}?{resolve}'.format(
                      endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                     id=self.stack_id),
+                     id=self.stack_id, resolve=resolve),
                  status_code=302,
                  headers=dict(
-                     location='{endpoint}/stacks/{name}/{id}'.format(
+                     location='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                          endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                         id=self.stack_id, name=self.stack_name))),
+                         id=self.stack_id, name=self.stack_name,
+                         resolve=resolve))),
             dict(method='GET',
-                 uri='{endpoint}/stacks/{name}/{id}'.format(
+                 uri='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                      endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                     id=self.stack_id, name=self.stack_name),
+                     id=self.stack_id, name=self.stack_name, resolve=resolve),
                  json={"stack": self.stack}),
             dict(method='DELETE',
                  uri='{endpoint}/stacks/{id}'.format(
@@ -180,20 +188,23 @@ class TestStack(base.TestCase):
             self.stack_id, self.stack_name, status='CREATE_COMPLETE')
         marker_qs = 'marker={e_id}&sort_dir=asc'.format(
             e_id=marker_event['id'])
+        resolve = 'resolve_outputs=False'
         self.register_uris([
             dict(method='GET',
-                 uri='{endpoint}/stacks/{id}'.format(
+                 uri='{endpoint}/stacks/{id}?{resolve}'.format(
                      endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                     id=self.stack_id),
+                     id=self.stack_id,
+                     resolve=resolve),
                  status_code=302,
                  headers=dict(
-                     location='{endpoint}/stacks/{name}/{id}'.format(
+                     location='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                          endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                         id=self.stack_id, name=self.stack_name))),
+                         id=self.stack_id, name=self.stack_name,
+                         resolve=resolve))),
             dict(method='GET',
-                 uri='{endpoint}/stacks/{name}/{id}'.format(
+                 uri='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                      endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                     id=self.stack_id, name=self.stack_name),
+                     id=self.stack_id, name=self.stack_name, resolve=resolve),
                  json={"stack": self.stack}),
             dict(method='GET',
                  uri='{endpoint}/stacks/{id}/events?{qs}'.format(
@@ -218,9 +229,9 @@ class TestStack(base.TestCase):
                          status='DELETE_COMPLETE'),
                  ]}),
             dict(method='GET',
-                 uri='{endpoint}/stacks/{id}'.format(
+                 uri='{endpoint}/stacks/{id}?{resolve}'.format(
                      endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                     id=self.stack_id, name=self.stack_name),
+                     id=self.stack_id, resolve=resolve),
                  status_code=404),
         ])
 
@@ -234,20 +245,22 @@ class TestStack(base.TestCase):
             self.stack_id, self.stack_name, status='CREATE_COMPLETE')
         marker_qs = 'marker={e_id}&sort_dir=asc'.format(
             e_id=marker_event['id'])
+        resolve = 'resolve_outputs=False'
         self.register_uris([
             dict(method='GET',
-                 uri='{endpoint}/stacks/{id}'.format(
+                 uri='{endpoint}/stacks/{id}?{resolve}'.format(
                      endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                     id=self.stack_id),
+                     id=self.stack_id, resolve=resolve),
                  status_code=302,
                  headers=dict(
-                     location='{endpoint}/stacks/{name}/{id}'.format(
+                     location='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                          endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                         id=self.stack_id, name=self.stack_name))),
+                         id=self.stack_id, name=self.stack_name,
+                         resolve=resolve))),
             dict(method='GET',
-                 uri='{endpoint}/stacks/{name}/{id}'.format(
+                 uri='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                      endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                     id=self.stack_id, name=self.stack_name),
+                     id=self.stack_id, name=self.stack_name, resolve=resolve),
                  json={"stack": self.stack}),
             dict(method='GET',
                  uri='{endpoint}/stacks/{id}/events?{qs}'.format(
@@ -272,18 +285,19 @@ class TestStack(base.TestCase):
                          status='DELETE_COMPLETE'),
                  ]}),
             dict(method='GET',
-                 uri='{endpoint}/stacks/{id}'.format(
+                 uri='{endpoint}/stacks/{id}?resolve_outputs=False'.format(
                      endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                     id=self.stack_id, name=self.stack_name),
+                     id=self.stack_id),
                  status_code=302,
                  headers=dict(
-                     location='{endpoint}/stacks/{name}/{id}'.format(
+                     location='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                          endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                         id=self.stack_id, name=self.stack_name))),
+                         id=self.stack_id, name=self.stack_name,
+                         resolve=resolve))),
             dict(method='GET',
-                 uri='{endpoint}/stacks/{name}/{id}'.format(
+                 uri='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                      endpoint=fakes.ORCHESTRATION_ENDPOINT,
-                     id=self.stack_id, name=self.stack_name),
+                     id=self.stack_id, name=self.stack_name, resolve=resolve),
                  json={"stack": failed_stack}),
         ])
 
@@ -305,8 +319,6 @@ class TestStack(base.TestCase):
                 validate=dict(
                     json={
                         'disable_rollback': False,
-                        'environment': {},
-                        'files': {},
                         'parameters': {},
                         'stack_name': self.stack_name,
                         'tags': self.stack_tag,
@@ -353,8 +365,6 @@ class TestStack(base.TestCase):
                 validate=dict(
                     json={
                         'disable_rollback': False,
-                        'environment': {},
-                        'files': {},
                         'parameters': {},
                         'stack_name': self.stack_name,
                         'tags': self.stack_tag,
@@ -411,11 +421,11 @@ class TestStack(base.TestCase):
                 validate=dict(
                     json={
                         'disable_rollback': False,
-                        'environment': {},
-                        'files': {},
                         'parameters': {},
+                        'tags': self.stack_tag,
                         'template': fakes.FAKE_TEMPLATE_CONTENT,
-                        'timeout_mins': 60})),
+                        'timeout_mins': 60}),
+                json={}),
             dict(
                 method='GET',
                 uri='{endpoint}/stacks/{name}'.format(
@@ -435,6 +445,7 @@ class TestStack(base.TestCase):
         ])
         self.cloud.update_stack(
             self.stack_name,
+            tags=self.stack_tag,
             template_file=test_template.name)
 
         self.assert_calls()
@@ -465,11 +476,11 @@ class TestStack(base.TestCase):
                 validate=dict(
                     json={
                         'disable_rollback': False,
-                        'environment': {},
-                        'files': {},
                         'parameters': {},
+                        'tags': self.stack_tag,
                         'template': fakes.FAKE_TEMPLATE_CONTENT,
-                        'timeout_mins': 60})),
+                        'timeout_mins': 60}),
+                json={}),
             dict(
                 method='GET',
                 uri='{endpoint}/stacks/{name}/events?{qs}'.format(
@@ -501,6 +512,7 @@ class TestStack(base.TestCase):
         ])
         self.cloud.update_stack(
             self.stack_name,
+            tags=self.stack_tag,
             template_file=test_template.name,
             wait=True)
 
