@@ -11,7 +11,7 @@
 # under the License.
 
 import mock
-import testtools
+from openstack.tests.unit import base
 
 from openstack.key_manager.v1 import secret
 
@@ -35,17 +35,16 @@ EXAMPLE = {
 }
 
 
-class TestSecret(testtools.TestCase):
+class TestSecret(base.TestCase):
 
     def test_basic(self):
         sot = secret.Secret()
         self.assertIsNone(sot.resource_key)
         self.assertEqual('secrets', sot.resources_key)
         self.assertEqual('/secrets', sot.base_path)
-        self.assertEqual('key-manager', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
@@ -58,7 +57,9 @@ class TestSecret(testtools.TestCase):
                               "updated": "updated",
                               "expiration": "expiration",
                               "sort": "sort",
-                              "algorithm": "alg"},
+                              "algorithm": "alg",
+                              "limit": "limit",
+                              "marker": "marker"},
                              sot._query_mapping._mapping)
 
     def test_make_it(self):
@@ -90,17 +91,16 @@ class TestSecret(testtools.TestCase):
         rv.json = mock.Mock(return_value=return_body)
         sess.get = mock.Mock(return_value=rv)
 
-        sot.get(sess)
+        sot.fetch(sess)
 
-        sess.get.assert_called_once_with("secrets/id",
-                                         endpoint_filter=sot.service)
+        sess.get.assert_called_once_with("secrets/id")
 
     def _test_payload(self, sot, metadata, content_type):
         content_type = "some/type"
-        sot = secret.Secret(id="id", payload_content_type=content_type)
 
         metadata_response = mock.Mock()
-        metadata_response.json = mock.Mock(return_value=metadata)
+        # Use copy because the dict gets consumed.
+        metadata_response.json = mock.Mock(return_value=metadata.copy())
 
         payload_response = mock.Mock()
         payload = "secret info"
@@ -109,11 +109,11 @@ class TestSecret(testtools.TestCase):
         sess = mock.Mock()
         sess.get = mock.Mock(side_effect=[metadata_response, payload_response])
 
-        rv = sot.get(sess)
+        rv = sot.fetch(sess)
 
         sess.get.assert_has_calls(
-            [mock.call("secrets/id", endpoint_filter=sot.service),
-             mock.call("secrets/id/payload", endpoint_filter=sot.service,
+            [mock.call("secrets/id",),
+             mock.call("secrets/id/payload",
                        headers={"Accept": content_type})])
 
         self.assertEqual(rv.payload, payload)
